package dev.w9r.config

import org.springframework.boot.actuate.endpoint.annotation.ReadOperation
import org.springframework.boot.actuate.endpoint.annotation.WriteOperation
import org.springframework.boot.actuate.endpoint.web.annotation.WebEndpoint
import org.springframework.boot.availability.AvailabilityChangeEvent
import org.springframework.boot.availability.ReadinessState
import org.springframework.context.ApplicationContext

@WebEndpoint(id = "statechange")
class StateChangeEndpoint(private val context: ApplicationContext) {
    private var state = State.RUNNING

    @ReadOperation
    fun getState(): String {
        return "{ \"state\": \"$state\" }"
    }

    @WriteOperation
    fun setState(): String {
        when (state) {
            State.RUNNING -> {
                AvailabilityChangeEvent.publish(context, ReadinessState.REFUSING_TRAFFIC)
                state = State.STOPPED
            }

            State.STOPPED -> {
                AvailabilityChangeEvent.publish(context, ReadinessState.ACCEPTING_TRAFFIC)
                state = State.RUNNING
            }
        }
        return "{ \"state\": \"$state\" }"
    }

    private enum class State {
        RUNNING, STOPPED
    }
}
